/*
    rfinger.c
    Copyright 1997 by Richard Dawe

    Rich's Finger client (based on RFC 742)

    ---

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* Include files */
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>

#include "common.h"     /* Common defines   */
#include "misc.h"       /* Common functions */

/* Defines */
#define APP_NAME        "rfinger"
#define APP_AUTHOR      "Richard Dawe"
#define APP_YEAR        "1997"
#define APP_VERSION     "1.0"

/* Function prototypes */
void msg_copyright (void);
void finger(int argc, char *argv[]);

/* --------
   - main -
   -------- */

int main (int argc, char *argv[])
{
    /* Copyright message */
    msg_copyright();

    /* Finger! */
    finger(argc, argv);

    /* OK */
    return(0);
}

/* -----------------
   - msg_copyright -
   ----------------- */

void msg_copyright (void)
{
    printf("%s v%s Copyright %s by %s\n", APP_NAME, APP_VERSION, APP_YEAR, APP_AUTHOR);
    printf("This program is distributed under the terms of the GNU General Public\nLicense - see the file LICENSE for details.\n\n");
}

/* ----------
   - finger -
   ---------- */

void finger(int argc, char *argv[])
{
    int fsock;
    struct sockaddr_in sin;

    struct protoent *tcpproto;
    struct servent *fingerserv;
    struct hostent *fhost;

    char userhost[256];
    char *user, *host;

    char msg_log[256];
    char msg[MSG_MAX_LEN];

    char buffer[BUFFER_LEN];            /* Buffer for recv's            */
    char *buffer_pos;                   /* Current position in buffer   */
    int buffer_read;                    /* Number of bytes read         */

    char *tptr;

    /* Find out the host/user bizness */
    if (argc < 2) die(APP_NAME, "No host specified!", 1);

    tptr = user = host = NULL;
    memset(userhost, 0, sizeof(userhost));
    strcpy(userhost, argv[1]);

    tptr = strchr(userhost, '@');
    if (tptr != NULL)
    {
        user = userhost;
        host = tptr + 1;
        *tptr = '\0';
    }
    else
        host = userhost;

    /* Look up the hostname */
    fhost = gethostbyname(host);
    if (fhost == NULL)
    {
        sprintf(msg_log, "Unable to resolve host name '%s'!", host);
        die(APP_NAME, msg_log, 1);
    }

    /* Create the socket */
    tcpproto = getprotobyname("tcp");
    fsock = socket(AF_INET, SOCK_STREAM, tcpproto->p_proto);

    fingerserv = getservbyname("finger", "tcp");
    if (fingerserv == NULL)
        die(APP_NAME, "Unable to obtain finger's port number!", 1);

    memset(&sin, 0, sizeof(sin));
    sin.sin_family = AF_INET;
    sin.sin_addr.s_addr = ( (struct in_addr *) fhost->h_addr)->s_addr;
    sin.sin_port = fingerserv->s_port;

    /* Connect */
    if (connect(fsock, (struct sockaddr *) &sin, sizeof(sin)) == -1)
        die(APP_NAME, "Connect failed!", 1);

    if (user != NULL)
        printf("[%s@%s]\n", user, host);
    else
        printf("[%s]\n", host);

    /* Request the appropriate info */
    memset(msg, 0, sizeof(msg));
    if (user != NULL) strcpy(msg, user);
    strcat(msg, CRLF);

    if (send(fsock, msg, strlen(msg), 0) == -1)
        die(APP_NAME, "Unable to send finger request!", 1);

    /* Read the returning data */
    memset(buffer, 0, sizeof(buffer));
    buffer_pos = buffer;

    while( (buffer_read = recv( fsock, buffer_pos,
                 (BUFFER_LEN - (unsigned long) buffer_pos + (unsigned long) buffer),
                 0
               )) > 0)
    {
        buffer_pos += buffer_read;
    }

    /* Print it! */
    printf("%s\n", buffer);
}
